@@ -1,9 +1,12 @@
 package dyld
 
+//go:generate stringer -type=dyld_section_location_kind -trimprefix=dyld_section_location_ -output prebuilt_string.go
+
 import (
 	"crypto/sha1"
 	"fmt"
 	"strings"
+	"text/tabwriter"
 
 	"github.com/blacktop/go-macho/types"
 	"github.com/olekukonko/tablewriter"
@@ -40,11 +43,11 @@ func (l LoaderRef) String() string {
 	if l.IsApp() {
 		typ = ", type: app"
 	}
-	mssing_weak_image := ""
+	missing_weak_image := ""
 	if l.IsMissingWeakImage() {
-		mssing_weak_image = " (missing weak image)"
+		missing_weak_image = " (missing weak image)"
 	}
-	return fmt.Sprintf("index: %d%s%s", l.Index(), typ, mssing_weak_image)
+	return fmt.Sprintf("index: %d%s%s", l.Index(), typ, missing_weak_image)
 }
 
 type Loader struct {
@@ -61,6 +64,9 @@ type Loader struct {
 	// pre2022Binary      :  1,
 	// isPremapped        :  1,  // mapped by exclave core
 	// isVersion2         :  1,  // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
+	// unknown1           :  1,  // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
+	// unknown2           :  1,  // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
+	// unknown3           :  1,  // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
 	// padding            :  6;
 	Ref LoaderRef
 	// Unk [12]uint16
@@ -99,6 +105,18 @@ func (l Loader) IsPremapped() bool {
 func (l Loader) IsVersion2() bool {
 	return types.ExtractBits(uint64(l.Info), 10, 1) != 0
 }
+func (l Loader) Unknown1() bool {
+	return types.ExtractBits(uint64(l.Info), 11, 1) != 0
+}
+func (l Loader) Unknown2() bool {
+	return types.ExtractBits(uint64(l.Info), 12, 1) != 0
+}
+func (l Loader) Unknown3() bool {
+	return types.ExtractBits(uint64(l.Info), 13, 1) != 0
+}
+func (l Loader) Padding() uint8 {
+	return uint8(types.ExtractBits(uint64(l.Info), 14, 2))
+}
 
 func (l Loader) String() string {
 	var out []string
@@ -137,6 +155,15 @@ func (l Loader) String() string {
 	if l.IsVersion2() {
 		out = append(out, "v2")
 	}
+	if l.Unknown1() {
+		out = append(out, "unk1")
+	}
+	if l.Unknown2() {
+		out = append(out, "unk2")
+	}
+	if l.Unknown3() {
+		out = append(out, "unk3")
+	}
 	return fmt.Sprintf("%s, ref: %s", strings.Join(out, "|"), l.Ref)
 }
 
@@ -146,7 +173,8 @@ const (
 	KindNormal   DependentKind = 0
 	KindWeakLink DependentKind = 1
 	KindReexport DependentKind = 2
-	KindUpward   DependentKind = 3
+	KindUpward   DependentKind = 4
+	KindDelayed  DependentKind = 8
 )
 
 func (k DependentKind) String() string {
@@ -154,11 +182,13 @@ func (k DependentKind) String() string {
 	case KindNormal:
 		return "regular"
 	case KindWeakLink:
-		return "weak link"
+		return "weak-link"
 	case KindReexport:
-		return "reexport"
+		return "re-export"
 	case KindUpward:
 		return "upward"
+	case KindDelayed:
+		return "delay-init"
 	default:
 		return fmt.Sprintf("unknown %d", k)
 	}
@@ -330,6 +360,46 @@ type dependent struct {
 	Kind DependentKind
 }
 
+type dyld_section_location_kind uint32
+
+const (
+	// TEXT:
+	dyld_section_location_text_swift5_protos dyld_section_location_kind = iota
+	dyld_section_location_text_swift5_proto
+	dyld_section_location_text_swift5_types
+	dyld_section_location_text_swift5_replace
+	dyld_section_location_text_swift5_replace2
+	dyld_section_location_text_swift5_ac_funcs
+
+	// DATA*:
+	dyld_section_location_objc_image_info
+	dyld_section_location_data_sel_refs
+	dyld_section_location_data_msg_refs
+	dyld_section_location_data_class_refs
+	dyld_section_location_data_super_refs
+	dyld_section_location_data_protocol_refs
+	dyld_section_location_data_class_list
+	dyld_section_location_data_non_lazy_class_list
+	dyld_section_location_data_stub_list
+	dyld_section_location_data_category_list
+	dyld_section_location_data_category_list2
+	dyld_section_location_data_non_lazy_category_list
+	dyld_section_location_data_protocol_list
+	dyld_section_location_data_objc_fork_ok
+	dyld_section_location_data_raw_isa
+
+	// Note, always add new entries before this
+	dyld_section_location_count
+)
+
+type SectionLocations struct {
+	Version uint32 // = 1;
+	Flags   uint32 // = 0;
+
+	Offsets [dyld_section_location_count]uint64
+	Sizes   [dyld_section_location_count]uint64
+}
+
 type prebuiltLoaderHeader struct {
 	PathOffset                     uint16
 	DependentLoaderRefsArrayOffset uint16 // offset to array of LoaderRef
@@ -367,6 +437,10 @@ type prebuiltLoaderHeader struct {
 	OverrideBindTargetRefsOffset uint32
 	OverrideBindTargetRefsCount  uint32
 
+	_ uint32 // padding
+
+	Sections SectionLocations
+
 	// followed by:
 	//  path chars
 	//  dep kind array
@@ -375,6 +449,41 @@ type prebuiltLoaderHeader struct {
 	//  bind targets
 }
 
+func (hdr prebuiltLoaderHeader) HasInitializers() bool {
+	return types.ExtractBits(uint64(hdr.Info), 0, 1) != 0
+}
+func (hdr prebuiltLoaderHeader) IsOverridable() bool {
+	return types.ExtractBits(uint64(hdr.Info), 1, 1) != 0
+}
+func (hdr prebuiltLoaderHeader) SupportsCatalyst() bool {
+	return types.ExtractBits(uint64(hdr.Info), 2, 1) != 0
+}
+func (hdr prebuiltLoaderHeader) IsCatalystOverride() bool {
+	return types.ExtractBits(uint64(hdr.Info), 3, 1) != 0
+}
+func (hdr prebuiltLoaderHeader) RegionsCount() uint16 {
+	return uint16(types.ExtractBits(uint64(hdr.Info), 4, 12))
+}
+func (hdr prebuiltLoaderHeader) GetInfo() string {
+	var out []string
+	if hdr.HasInitializers() {
+		out = append(out, "initializers")
+	}
+	if hdr.IsOverridable() {
+		out = append(out, "overridable")
+	}
+	if hdr.SupportsCatalyst() {
+		out = append(out, "catalyst")
+	}
+	if hdr.IsCatalystOverride() {
+		out = append(out, "catalyst_override")
+	}
+	if hdr.RegionsCount() > 0 {
+		out = append(out, fmt.Sprintf("regions=%d", hdr.RegionsCount()))
+	}
+	return strings.Join(out, "|")
+}
+
 // ObjCBinaryInfo stores information about the layout of the objc sections in a binary,
 // as well as other properties relating to the objc information in there.
 type ObjCBinaryInfo struct {
@@ -463,6 +572,8 @@ func (o ObjCBinaryInfo) String() string {
 
 type PrebuiltLoader struct {
 	Loader
+	UUID                        types.UUID
+	Unknown                     [2]uint32 // FIXME: when dyld src is out for iOS 18.0/macOS 15.0
 	Header                      prebuiltLoaderHeader
 	Path                        string
 	AltPath                     string
@@ -478,40 +589,6 @@ type PrebuiltLoader struct {
 	ObjcSelectorFixups          []BindTargetRef
 }
 
-func (pl PrebuiltLoader) HasInitializers() bool {
-	return types.ExtractBits(uint64(pl.Info), 0, 1) != 0
-}
-func (pl PrebuiltLoader) IsOverridable() bool {
-	return types.ExtractBits(uint64(pl.Info), 1, 1) != 0
-}
-func (pl PrebuiltLoader) SupportsCatalyst() bool {
-	return types.ExtractBits(uint64(pl.Info), 2, 1) != 0
-}
-func (pl PrebuiltLoader) IsCatalystOverride() bool {
-	return types.ExtractBits(uint64(pl.Info), 3, 1) != 0
-}
-func (pl PrebuiltLoader) RegionsCount() uint16 {
-	return uint16(types.ExtractBits(uint64(pl.Info), 4, 12))
-}
-func (pl PrebuiltLoader) GetInfo() string {
-	var out []string
-	if pl.HasInitializers() {
-		out = append(out, "initializers")
-	}
-	if pl.IsOverridable() {
-		out = append(out, "overridable")
-	}
-	if pl.SupportsCatalyst() {
-		out = append(out, "catalyst")
-	}
-	if pl.IsCatalystOverride() {
-		out = append(out, "catalyst_override")
-	}
-	if pl.RegionsCount() > 0 {
-		out = append(out, fmt.Sprintf("regions=%d", pl.RegionsCount()))
-	}
-	return strings.Join(out, "|")
-}
 func (pl PrebuiltLoader) GetFileOffset(vmoffset uint64) uint64 {
 	for _, region := range pl.Regions {
 		if vmoffset >= region.VMOffset() && vmoffset < region.VMOffset()+uint64(region.FileSize) {
@@ -552,8 +629,8 @@ func (pl PrebuiltLoader) String(f *File) string {
 		// }
 	}
 	out += fmt.Sprintf("Loader:        %s\n", pl.Loader)
-	if len(pl.GetInfo()) > 0 {
-		out += fmt.Sprintf("Info:          %s\n", pl.GetInfo())
+	if len(pl.Header.GetInfo()) > 0 {
+		out += fmt.Sprintf("Info:          %s\n", pl.Header.GetInfo())
 	}
 	if pl.Header.ExportsTrieLoader.Size > 0 {
 		out += fmt.Sprintf("ExportsTrie:   off=%#08x, sz=%#x\n", pl.GetFileOffset(pl.Header.ExportsTrieLoader.Offset), pl.Header.ExportsTrieLoader.Size)
@@ -562,7 +639,7 @@ func (pl PrebuiltLoader) String(f *File) string {
 		out += fmt.Sprintf("FixupsLoadCmd: off=%#08x\n", pl.Header.FixupsLoadCommandOffset)
 	}
 	if len(pl.Regions) > 0 {
-		out += "\nRegions:\n"
+		out += "\nRegions:\n\n"
 		tableString := &strings.Builder{}
 		rdata := [][]string{}
 		for _, rg := range pl.Regions {
@@ -584,22 +661,33 @@ func (pl PrebuiltLoader) String(f *File) string {
 		table.Render()
 		out += tableString.String()
 	}
+	out += "\nSections:\n"
+	buf := &strings.Builder{}
+	w := tabwriter.NewWriter(buf, 0, 0, 1, ' ', 0)
+	for idx, off := range pl.Header.Sections.Offsets {
+		if off == 0 {
+			continue
+		}
+		fmt.Fprintf(w, "    %s:\toff=%#x\tsz=%d\n", dyld_section_location_kind(idx), off, pl.Header.Sections.Sizes[idx])
+	}
+	w.Flush()
+	out += buf.String()
 	if len(pl.Dependents) > 0 {
 		out += "\nDependents:\n"
 		for _, dp := range pl.Dependents {
-			out += fmt.Sprintf("\t%-10s) %s\n", dp.Kind, dp.Name)
+			out += fmt.Sprintf("    %-10s) %s\n", dp.Kind, dp.Name)
 		}
 	}
 	if len(pl.BindTargets) > 0 {
 		out += "\nBindTargets:\n"
 		for _, bt := range pl.BindTargets {
-			out += fmt.Sprintf("  %s\n", bt.String(f))
+			out += fmt.Sprintf("    %s\n", bt.String(f))
 		}
 	}
 	if len(pl.OverrideBindTargets) > 0 {
 		out += "\nOverride BindTargets:\n"
 		for _, bt := range pl.OverrideBindTargets {
-			out += fmt.Sprintf("  %s\n", bt.String(f))
+			out += fmt.Sprintf("    %s\n", bt.String(f))
 		}
 	}
 	if pl.ObjcFixupInfo != nil {
@@ -609,13 +697,13 @@ func (pl PrebuiltLoader) String(f *File) string {
 	if len(pl.ObjcCanonicalProtocolFixups) > 0 {
 		out += "ObjC Canonical ProtocolFixups:\n"
 		for _, fixup := range pl.ObjcCanonicalProtocolFixups {
-			out += fmt.Sprintf("  %t\n", fixup)
+			out += fmt.Sprintf("    %t\n", fixup)
 		}
 	}
 	if len(pl.ObjcSelectorFixups) > 0 {
 		out += "\nObjC SelectorFixups:\n"
 		for _, bt := range pl.ObjcSelectorFixups {
-			out += fmt.Sprintf("  %s\n", bt.String(f))
+			out += fmt.Sprintf("    %s\n", bt.String(f))
 		}
 	}
 
